#!/bin/zsh
#
# script to convert hwraidtools output to CDB structure describing devices
#
# see https://twiki.cern.ch/twiki/bin/view/FIOgroup/TsiCDBBlockDevices
# see http://cern.ch/ahorvath/hwraidtools/
#
# Andras.Horvath@cern.ch, 2008

set -o shwordsplit

ROOT=/system/blockdevices/hwraid

i=0 # raid unit index (global)
c=0 # card index (global)
lastcard="" # the last card seen in 'hwraidman info', e.g. 'c0'

declare -a physdev

### --------------------------------------------

echo "\"$ROOT\" = nlist ("

tr ',' ' ' | while read hwtype ctrlr unit raid stripe size portlist devname state; do
	unit=${unit#u}

	# count RAID cards..
	# FIXME: what's the index in /hardware/cards/raid exactly?
	if [[ ! -z "$lastcard" ]] && [[ "$lastcard" != "$ctrlr" ]]; then
		(( c++ ))
	fi
	lastcard=$ctrlr

	# in some cards SPARE drives may be reported without a unit number.
	# so let's give them one
	if [[ "$raid" == "SPARE" ]] && [[ "$unit" == "-" ]]; then
		unit=$(( 900 + $i ))
	fi

	echo "    \"_$i\", nlist ( \"card\", \"/hardware/cards/raid/_$c\","
	echo "                  \"unit\", \"$unit\","	
	echo "                  \"raid_level\", \"${raid//-/}\","	
	# some cards don't report a stripe size
	# CDB would complain
	if [[ "$stripe" != "-" ]]; then 
		echo "                  \"stripe_size\", ${stripe%K},"	
	fi
	echo "                  \"device_list\", list ("
	portlist=${portlist//:/ }
	for p in $portlist; do
		# Areca cards number ports from one. Others from zero. ....
		if [[ "$hwtype" == "areca" ]] ; then
			(( p-- ))
		fi
		echo "                      \"raid/_$c/ports/_$p\","
	done
	echo "                  )" # device_list list
	echo "    )," # unit nlist

	physdev[$(( $i + 1 ))]=$devname

	(( i++ ))
done

echo ");" # $ROOT nlist 
echo

for (( j=1; j<=$i; j++ )); do
	pd=${physdev[$j]}
	if [[ ! -z "$pd" ]] && [[ "$pd" != "-" ]]; then
		echo "\"/system/blockdevices/physical_devs/$pd/device_path\" = \"hwraid/_$(( $j - 1 ))\";"
		echo "\"/system/blockdevices/physical_devs/$pd/label\" = \"none\";"
	fi
done

# EOT
